
// (c) COPYRIGHT URI/MIT 1997-98
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors: reza (Reza Nekovei)

// This is the main CGI program which extracts the requested variable from a
// FreeFrom data file. It generates the DDS in-memory from both the FreeForm
// format file and external file and uses it to locate the variable to process.
//
// ReZa 6/10/97

// $Log: ff_dods.cc,v $
// Revision 1.9  1999/01/05 00:49:11  jimg
// Added ce functions for data_time. Removed some unnecessary ce functions (the
// selection functions for synthesized variables don't need to be registered).
//
// Revision 1.8  1998/12/22 22:30:17  jimg
// Added fix to set the filename of the dataset in the DDS.
//
// Revision 1.7  1998/11/10 03:47:22  jimg
// Add the new DODS_Date aand DODS_JDate projection functions. This is here to
// prototype the CS design.
//
// Revision 1.6  1998/09/17 17:42:12  jimg
// Added installation of date_func in DDS.
//
// Revision 1.5  1998/08/12 21:21:11  jimg
// Massive changes from Reza. Compatible with the new FFND library
//
// Revision 1.4  1998/04/21 17:14:08  jimg
// Fixes for warnings, etc
//
// Revision 1.3  1998/04/16 18:11:24  jimg
// Sequence support added by Reza
//
//

#include "config_ff.h"

static char rcsid[] __unused__ ={"$Id: ff_dods.cc,v 1.9 1999/01/05 00:49:11 jimg Exp $"};

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#include <iostream.h>

#include "cgi_util.h"
#include "DDS.h"
#include "FFArray.h"
#include "FFGrid.h"
#include "DODSFilter.h"
#include "ce_functions.h"

long BufPtr = 0; // cache pointer
long BufSiz =0; // Cache size
char *BufVal = NULL; // cache buffer

extern bool read_descriptors(DDS &das, const char * filename, String *err_msg);

int 
main(int argc, char * argv[])
{
    DDS dds;
    dds.add_function("date", func_date);
    dds.add_function("DODS_JDate", proj_dods_jdate);
    dds.add_function("DODS_Date", proj_dods_date);

    dds.add_function("time", func_time);
    dds.add_function("DODS_Time", proj_dods_time);

    dds.add_function("date_time", func_date_time);
    dds.add_function("DODS_Date_Time", proj_dods_date_time);

    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
	df.send_version_info();
	return 0;
    }

    // Set the filename. 12/22/98 jhrg
    dds.filename(df.get_dataset_name());

    // Read the dataset descriptor in memory
    String errMsg;
    if(!read_descriptors(dds, (const char *)df.get_dataset_name(), &errMsg)){
	ErrMsgT("ff_dods: could not read the DDS information");
	//client error message
	Error e(unknown_error, errMsg);
	set_mime_text(cout, dods_error);
	e.print(cout);
	return 1;
    }

    if (!df.read_ancillary_dds(dds))
	return 1;

    if (!df.send_data(dds, stdout))
	return 1;

    return 0;
}
